@@ -153,7 +153,7 @@ class ZLibCodec extends Codec<List<int>, List<int>> {
    * be predicted with good accuracy; the data can then be compressed better
    * than with the default empty dictionary.
    */
-  final List<int> dictionary;
+  final List<int>? dictionary;
 
   ZLibCodec(
       {this.level: ZLibOption.defaultLevel,
@@ -264,7 +264,7 @@ class GZipCodec extends Codec<List<int>, List<int>> {
    * be predicted with good accuracy; the data can then be compressed better
    * than with the default empty dictionary.
    */
-  final List<int> dictionary;
+  final List<int>? dictionary;
 
   /**
    * When true, deflate generates raw data with no zlib header or trailer, and
@@ -371,7 +371,7 @@ class ZLibEncoder extends Converter<List<int>, List<int>> {
    * be predicted with good accuracy; the data can then be compressed better
    * than with the default empty dictionary.
    */
-  final List<int> dictionary;
+  final List<int>? dictionary;
 
   /**
    * When true, deflate generates raw data with no zlib header or trailer, and
@@ -414,7 +414,7 @@ class ZLibEncoder extends Converter<List<int>, List<int>> {
     if (sink is! ByteConversionSink) {
       sink = new ByteConversionSink.from(sink);
     }
-    return new _ZLibEncoderSink(
+    return new _ZLibEncoderSink._(
         sink, gzip, level, windowBits, memLevel, strategy, dictionary, raw);
   }
 }
@@ -440,7 +440,7 @@ class ZLibDecoder extends Converter<List<int>, List<int>> {
    * be predicted with good accuracy; the data can then be compressed better
    * than with the default empty dictionary.
    */
-  final List<int> dictionary;
+  final List<int>? dictionary;
 
   /**
    * When true, deflate generates raw data with no zlib header or trailer, and
@@ -476,7 +476,7 @@ class ZLibDecoder extends Converter<List<int>, List<int>> {
     if (sink is! ByteConversionSink) {
       sink = new ByteConversionSink.from(sink);
     }
-    return new _ZLibDecoderSink(sink, windowBits, dictionary, raw);
+    return new _ZLibDecoderSink._(sink, windowBits, dictionary, raw);
   }
 }
 
@@ -494,7 +494,7 @@ abstract class RawZLibFilter {
     int windowBits: ZLibOption.defaultWindowBits,
     int memLevel: ZLibOption.defaultMemLevel,
     int strategy: ZLibOption.strategyDefault,
-    List<int> dictionary,
+    List<int>? dictionary,
     bool raw: false,
   }) {
     return _makeZLibDeflateFilter(
@@ -507,7 +507,7 @@ abstract class RawZLibFilter {
    */
   factory RawZLibFilter.inflateFilter({
     int windowBits: ZLibOption.defaultWindowBits,
-    List<int> dictionary,
+    List<int>? dictionary,
     bool raw: false,
   }) {
     return _makeZLibInflateFilter(windowBits, dictionary, raw);
@@ -527,7 +527,7 @@ abstract class RawZLibFilter {
    * The last call to [processed] should have [end] set to [:true:]. This will
    * make sure an 'end' packet is written on the stream.
    */
-  List<int> processed({bool flush: true, bool end: false});
+  List<int>? processed({bool flush: true, bool end: false});
 
   external static RawZLibFilter _makeZLibDeflateFilter(
       bool gzip,
@@ -535,11 +535,11 @@ abstract class RawZLibFilter {
       int windowBits,
       int memLevel,
       int strategy,
-      List<int> dictionary,
+      List<int>? dictionary,
       bool raw);
 
   external static RawZLibFilter _makeZLibInflateFilter(
-      int windowBits, List<int> dictionary, bool raw);
+      int windowBits, List<int>? dictionary, bool raw);
 }
 
 class _BufferSink extends ByteConversionSink {
@@ -552,7 +552,8 @@ class _BufferSink extends ByteConversionSink {
   void addSlice(List<int> chunk, int start, int end, bool isLast) {
     if (chunk is Uint8List) {
       Uint8List list = chunk;
-      builder.add(new Uint8List.view(list.buffer, start, end - start));
+      builder.add(new Uint8List.view(
+          list.buffer, list.offsetInBytes + start, end - start));
     } else {
       builder.add(chunk.sublist(start, end));
     }
@@ -562,14 +563,14 @@ class _BufferSink extends ByteConversionSink {
 }
 
 class _ZLibEncoderSink extends _FilterSink {
-  _ZLibEncoderSink(
+  _ZLibEncoderSink._(
       ByteConversionSink sink,
       bool gzip,
       int level,
       int windowBits,
       int memLevel,
       int strategy,
-      List<int> dictionary,
+      List<int>? dictionary,
       bool raw)
       : super(
             sink,
@@ -578,8 +579,8 @@ class _ZLibEncoderSink extends _FilterSink {
 }
 
 class _ZLibDecoderSink extends _FilterSink {
-  _ZLibDecoderSink(
-      ByteConversionSink sink, int windowBits, List<int> dictionary, bool raw)
+  _ZLibDecoderSink._(
+      ByteConversionSink sink, int windowBits, List<int>? dictionary, bool raw)
       : super(sink,
             RawZLibFilter._makeZLibInflateFilter(windowBits, dictionary, raw));
 }
@@ -597,8 +598,9 @@ class _FilterSink extends ByteConversionSink {
   }
 
   void addSlice(List<int> data, int start, int end, bool isLast) {
+    // TODO(40614): Remove once non-nullability is sound.
+    ArgumentError.checkNotNull(end, "end");
     if (_closed) return;
-    if (end == null) throw new ArgumentError.notNull("end");
     RangeError.checkValidRange(start, end, data.length);
     try {
       _empty = false;
@@ -606,8 +608,10 @@ class _FilterSink extends ByteConversionSink {
           _ensureFastAndSerializableByteData(data, start, end);
       _filter.process(bufferAndStart.buffer, bufferAndStart.start,
           end - (start - bufferAndStart.start));
-      List<int> out;
-      while ((out = _filter.processed(flush: false)) != null) {
+      List<int>? out;
+      while (true) {
+        final out = _filter.processed(flush: false);
+        if (out == null) break;
         _sink.add(out);
       }
     } catch (e) {
@@ -624,8 +628,9 @@ class _FilterSink extends ByteConversionSink {
     // message would not have a GZip frame (if compressed with GZip).
     if (_empty) _filter.process(const [], 0, 0);
     try {
-      List<int> out;
-      while ((out = _filter.processed(end: true)) != null) {
+      while (true) {
+        final out = _filter.processed(end: true);
+        if (out == null) break;
         _sink.add(out);
       }
     } catch (e) {
